-- LÖVE 0.10.x Splash Screen
-- by S0lll0s, vrld, zorg
-- changelog:
-- - v3 (zorg)
-- - - Made it a bit more "state-like" in that hump gs can now support it, making integration with actual games easier.
-- - - Changed background color to the one used on the forums.
-- - - Added "Made with" text as text object + shader shenanigans.
-- - - Added LÖVE logo image from wiki, with a mask for gradual drawing via shader.

local timer = require 'lib.timer'

local colors = {
	bg =     { 108, 190, 228, 255 },
	white =  { 255, 255, 255, 255 },
	blue =   {  39, 170, 225, 255 },
	pink =   { 231,  74, 153, 255 },
	shadow = {   0,   0,   0, 255 / 3 }
}

local heart, stripes, width, height, shader, logoshader, textshader, text, logo

local state = {}

function state.init()

	width, height = love.graphics.getDimensions()

	-- shader lightens everything except in a circular mask

	shader = love.graphics.newShader[[
	extern number radius;
	extern number blur;
	extern number lighten;
	extern number shadow;

	vec4 desat(vec4 color)
	{
		// roughly human luminance perception
		number g = dot(vec3(.299, .587, .114), color.rgb);
		return vec4(g,g,g,1.0) * lighten;
	}

	vec4 effect(vec4 color, Image canvas, vec2 tc, vec2 _)
	{
		// radial mask
		color = Texel(canvas, tc);
		number r = length((tc - vec2(.5)) * love_ScreenSize.xy);
		number s = smoothstep(radius-blur, radius+blur, r);
		color = desat(color) * s + color;

		// add shadow on lower diagonal along the circle
		number sr = 7. * (1. - smoothstep(-.1,.01,(1.-tc.x)-tc.y));
		s = (1. - pow(exp(-pow(radius-r, 2.) / sr),3.) * shadow);

		return color * s;
	}
	]]

	-- this shader applies a hand-drawn effect on the logo
	logoshader = love.graphics.newShader[[
	//Using the pen extern, only draw out pixels that have their color below a certain treshold.
	//Since pen will eventually equal 1.0, the full logo will be drawn out.

	extern number pen;
	extern Image mask;

	vec4 effect(vec4 color, Image logo, vec2 tc, vec2 sc)
	{
		number value = max(Texel(mask, tc).r, max(Texel(mask, tc).g, Texel(mask, tc).b));
		number alpha = Texel(mask, tc).a;


		//probably could be optimzied...
		if (alpha > 0.0) {
			if (pen < value) {
				discard;
			} else {
				return color * Texel(logo, tc);
			}
		}
	}
	]]

	-- this shader makes the text appear from left to right
	textshader = love.graphics.newShader[[
	extern number alpha;

	vec4 effect(vec4 color, Image logo, vec2 tc, vec2 sc)
	{
		vec2 sd = sc / love_ScreenSize.xy;

		if (sd.x > alpha) {
			discard;
		} else {
			return color * Texel(logo, tc);
		}
	}
	]]

	local ssend = shader.send
	getmetatable(shader).send = function(self, ...) pcall(ssend, self, ...) end

	local ssend2 = textshader.send
	getmetatable(textshader).send = function(self, ...) pcall(ssend2, self, ...) end

	local ssend3 = logoshader.send
	getmetatable(logoshader).send = function(self, ...) pcall(ssend3, self, ...) end

	shader:send('radius', width*height)
	shader:send('lighten', 0)
	shader:send('blur', 1)
	shader:send('shadow', .2)

	textshader:send('alpha', 0.0)

	logoshader:send('pen', 0.0)

	canvas = love.graphics.newCanvas()

	heart = {
		sprite = love.graphics.newImage("heart.png"),
		scale = 0,
		rot = 0
	}

	stripes = {
		rot = 0,
		height = 100,
		offset = -2 * width,
		radius = math.max(width, height),
		lighten = 0,
		shadow = 0,
	}

	text = {
		obj = love.graphics.newText(love.graphics.newFont("res/fnt/Handy_Andy.otf",22), "made with"),
		alpha = 0.0
	}

	logo = {
		sprite = love.graphics.newImage("Love-logo-512x256.png"),
		mask = love.graphics.newImage("Love-logo-512x256-mask.png"),
		pen = 0.0
	}
	logoshader:send('mask',logo.mask)

	timer.clear()
	timer.script(function(wait)

		-- roll in stripes
		timer.tween(0.5, stripes, {offset = 0})
		wait(0.2)

		timer.tween(0.4, stripes, {rot = -5 * math.pi / 18, height=height})
		wait(0.2)

		-- hackety hack: execute timer to update shader every frame
		local haenker = timer.every(0, function()
			shader:send('radius', stripes.radius)
			shader:send('lighten', stripes.lighten)
			shader:send('shadow',  stripes.shadow)
			textshader:send('alpha', text.alpha)
			logoshader:send('pen', logo.pen)
		end)

		-- focus the heart, desaturate the rest
		timer.tween(0.2, stripes, {radius = 170})
		timer.tween(0.4, stripes, {lighten = .06}, 'quad')
		wait(0.2)

		timer.tween(0.2, stripes, {radius = 70}, 'out-back')
		timer.tween(0.4, stripes, {shadow = .3}, "back")

		-- make the heart appear
		timer.tween(1.0, heart, {scale = 1}, "out-elastic", nil, 1.0, 0.3)

		-- write out the text
		timer.tween(1.5, text, {alpha = 1.0}, "linear")
		wait(1.0)

		-- draw out the logo, in parts
		timer.tween(0.125, logo, {pen = 0.125}, "linear", function() -- L
		timer.tween(0.125, logo, {pen = 0.250}, "linear", function() -- O
		timer.tween(0.125, logo, {pen = 0.375}, "linear", function() -- first dot on O
		timer.tween(0.125, logo, {pen = 0.500}, "linear", function() -- second dot on O
		timer.tween(0.125, logo, {pen = 0.625}, "linear", function() -- \
		timer.tween(0.125, logo, {pen = 0.750}, "linear", function() -- /
		timer.tween(0.125, logo, {pen = 0.875}, "linear", function() -- e
		timer.tween(0.125, logo, {pen = 1.000}, "linear")            -- (c)
		end) end) end) end) end) end) end)
		wait(2.0)

		timer.clear()
	end)
end

function state.update(dt)
	dt = dt / 1.75
	timer.update(dt)
end

function state.draw()
	canvas:renderTo(function()
		love.graphics.clear(colors.bg)

		if stripes.offset == 0 then
			love.graphics.setColor(colors.shadow)
			love.graphics.circle("fill", width / 2 + 2, height / 2 + 2, stripes.radius, stripes.radius * 80)
		end

		love.graphics.push()
		love.graphics.translate(width / 2, height / 2)

		love.graphics.push()
		love.graphics.rotate(stripes.rot)
		love.graphics.setColor(colors.pink)
		love.graphics.rectangle("fill", stripes.offset - width, -stripes.height, width * 2, stripes.height)
		love.graphics.setColor(colors.blue)
		love.graphics.rectangle("line", -width - stripes.offset, 0, width * 2, stripes.height)
		-- draw line for anti aliasing
		love.graphics.rectangle("fill", -width - stripes.offset, 0, width * 2, stripes.height)
		love.graphics.pop()

		love.graphics.setColor(colors.white)
		love.graphics.draw(heart.sprite, 0, 5, heart.rot, heart.scale, heart.scale, 43, 39)

		-- TODO: position text better
		love.graphics.setShader(textshader)
		love.graphics.draw(text.obj, -text.obj:getWidth() / 2, height / 6)

		-- TODO: position logo better
		love.graphics.setShader(logoshader)
		love.graphics.draw(logo.sprite, -width/4 + logo.mask:getWidth()/8, logo.mask:getHeight()/2, 0, 0.5, 0.5)
		love.graphics.setShader()

		love.graphics.pop()
	end)

	love.graphics.setShader(shader)
	love.graphics.draw(canvas, 0,0)
	love.graphics.setShader()
	--love.graphics.print(logo.pen,0,0)
end

-- Make it easier to actually use this in a project.

love.load = state.init
love.update = state.update
love.draw = state.draw
love.keypressed = state.init
